@@ -8,7 +8,7 @@ use strict;
 
 package XML::Compile::Schema::Instance;
 use vars '$VERSION';
-$VERSION = '0.55';
+$VERSION = '0.56';
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
 use XML::Compile::Schema::Specs;
@@ -82,10 +82,10 @@ sub _collectTypes($)
         or panic "requires schema element";
 
     my $xsd = $self->{xsd} = $schema->namespaceURI;
-    my $def = $self->{def} =
-       XML::Compile::Schema::Specs->predefinedSchema($xsd)
-         or error __x"schema namespace {namespace} not (yet) supported"
-                , namespace => $xsd;
+    my $def = $self->{def}
+       = XML::Compile::Schema::Specs->predefinedSchema($xsd)
+            or error __x"schema namespace {namespace} not (yet) supported"
+                  , namespace => $xsd;
 
     my $xsi = $self->{xsi} = $def->{uri_xsi};
     my $tns = $self->{tns} = $schema->getAttribute('targetNamespace') || '';
