@@ -8,7 +8,7 @@ use strict;
 
 package XML::Compile::Schema::Specs;
 use vars '$VERSION';
-$VERSION = '0.55';
+$VERSION = '0.56';
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
 
@@ -20,33 +20,17 @@ use XML::Compile::Schema::BuiltInTypes   qw/%builtin_types/;
 # the official meta-scheme: the scheme of the scheme.  These lists are
 # used to restrict the namespace to the specified, hiding all helper
 # types.
-my %builtin_public_1999 =
- ();
 
-my %builtin_public_2000 = %builtin_public_1999;
-
-my @builtin_public_2001 = qw/
- anySimpleType
- anyType
- anyURI
+my @builtin_common = qw/
  boolean
- base64binary
  byte
  date
- dateTime
- dayTimeDuration
  decimal
  double
  duration
- ENTITY
  ENTITIES
+ ENTITY
  float
- gDay
- gMonth
- gMonthDay
- gYear
- gYearMonth
- hexBinary
  ID
  IDREF
  IDREFS
@@ -56,15 +40,13 @@ my @builtin_public_2001 = qw/
  long
  Name
  NCName
+ negativeInteger
  NMTOKEN
  NMTOKENS
- negativeInteger
  nonNegativeInteger
  nonPositiveInteger
- normalizedString
- positiveInteger
- precissionDecimal
  NOTATION
+ positiveInteger
  QName
  short
  string
@@ -77,7 +59,48 @@ my @builtin_public_2001 = qw/
  yearMonthDuration
  /;
 
-my %builtin_public_2001 = map { ($_ => $_) } @builtin_public_2001;
+my @builtin_extra_1999 = qw/
+ binary
+ recurringDate
+ recurringDay
+ recurringDuration
+ timeDuration
+ timeInstant
+ timePeriod
+ uriReference
+ year
+ /;
+
+my @builtin_extra_2000 = (@builtin_extra_1999, qw/
+ anyType
+ CDATA
+ / );
+
+my @builtin_extra_2001  = qw/
+ anySimpleType
+ anyType
+ anyURI
+ base64binary
+ dateTime
+ dayTimeDuration
+ gDay
+ gMonth
+ gMonthDay
+ gYear
+ gYearMonth
+ hexBinary
+ normalizedString
+ precissionDecimal
+ /;
+
+my %builtin_public_1999 = map { ($_ => $_) }
+   @builtin_common, @builtin_extra_1999;
+
+my %builtin_public_2000 = map { ($_ => $_) }
+   @builtin_common, @builtin_extra_2000;
+
+my %builtin_public_2001 = map { ($_ => $_) }
+   @builtin_common, @builtin_extra_2001;
 
 my %sloppy_int_version =
  ( decimal            => 'double'
